@@ -39,22 +39,11 @@ def cast_tuple(val, depth):
 
 # helper classes
 
-class RMSNorm(nn.Module):
-    def __init__(self, dim, eps = 1e-8):
-        super().__init__()
-        self.scale = dim ** -0.5
-        self.eps = eps
-        self.g = nn.Parameter(torch.ones(dim))
-
-    def forward(self, x):
-        norm = torch.norm(x, dim = -1, keepdim = True) * self.scale
-        return x / norm.clamp(min = self.eps) * self.g
-
 class PreNorm(nn.Module):
     def __init__(self, dim, fn):
         super().__init__()
         self.fn = fn
-        self.norm = RMSNorm(dim)
+        self.norm = nn.LayerNorm(dim)
 
     def forward(self, x, *args, **kwargs):
         x = self.norm(x)
@@ -64,8 +53,8 @@ class PreNormCross(nn.Module):
     def __init__(self, dim, fn):
         super().__init__()
         self.fn = fn
-        self.norm = RMSNorm(dim)
-        self.norm_context = RMSNorm(dim)
+        self.norm = nn.LayerNorm(dim)
+        self.norm_context = nn.LayerNorm(dim)
 
     def forward(self, x, context, *args, **kwargs):
         x = self.norm(x)
@@ -696,7 +685,7 @@ class Alphafold2(nn.Module):
         dim_distance_pred = constants.DISTOGRAM_BUCKETS if not predict_real_value_distances else 2   # 2 for predicting mean and standard deviation values of real-value distance
 
         self.to_distogram_logits = nn.Sequential(
-            RMSNorm(dim),
+            nn.LayerNorm(dim),
             nn.Sequential(
                 nn.Linear(dim, dim * (num_backbone_atoms ** 2)),
                 Rearrange('b h w c -> b c h w'),
@@ -737,6 +726,9 @@ class Alphafold2(nn.Module):
                     fourier_features = 2
                 )
 
+        # aux confidence measure
+        self.lddt_linear = nn.Linear(structure_module_dim, 1)
+
     def forward(
         self,
         seq,
@@ -749,7 +741,8 @@ class Alphafold2(nn.Module):
         templates_coors = None,
         templates_sidechains = None,
         embedds = None,
-        return_trunk = False
+        return_trunk = False,
+        return_confidence = False
     ):
         n, device = seq.shape[1], seq.device
         n_range = torch.arange(n, device = device)
@@ -784,11 +777,19 @@ class Alphafold2(nn.Module):
 
             msa_shape = m.shape
             m = rearrange(m, 'b m n d -> b (m n) d')
+            
+            # get msa_mask to all ones if none was passed
+            msa_mask = default(msa_mask, torch.ones_like(msa).bool())
 
         elif exists(embedds):
             m = self.embedd_project(embedds)
-            m = rearrange(m, 'b i d -> b i () d') + rearrange(m, 'b j d -> b () j d')
+            m += self.msa_pos_emb(torch.arange(embedds.shape[-2], device = device))[None, None, ...]
+
+            msa_shape = m.shape
             m = rearrange(m, 'b m n d -> b (m n) d')
+            
+            # get msa_mask to all ones if none was passed
+            msa_mask = default(msa_mask, torch.ones_like(embedds[..., -1]).bool())
 
         if exists(msa_mask):
             msa_mask = rearrange(msa_mask, 'b m n -> b (m n)')
@@ -955,4 +956,7 @@ class Alphafold2(nn.Module):
         if self.return_aux_logits:
             return coords, ret
 
+        if return_confidence:
+            return coords, self.lddt_linear(x.float())
+
         return coords
