@@ -214,19 +214,74 @@ def coords2pdb(seq, coords, cloud_mask, prefix="", name="af2_struct.pdb"):
     pred.to_pdb(prefix+name)
 
 
-# sidechainnet utils
+# sidechainnet / other data utils
 
-def scn_cloud_mask(scn_seq, boolean=True):
+def get_esm_embedd(seq, embedd_model, batch_converter):
+    """ Returns the ESM embeddings for a protein. 
+        Inputs: 
+        * seq: (L,) tensor of ints (in sidechainnet int-char convention)
+        * embedd_model: ESM model
+        * batch_converter: ESM batch converter
+    """
+    str_seq = "".join([VOCAB._int2char[x]for x in seq.cpu().numpy()])
+    batch_labels, batch_strs, batch_tokens = batch_converter( [(0, str_seq)] )
+    with torch.no_grad():
+        results = embedd_model(batch_tokens, repr_layers=[33], return_contacts=False)
+    return results["representations"][33].to(seq.device)
+
+
+def get_all_protein_ids(dataloader, verbose=False):
+    """ Given a sidechainnet dataloader for a CASP version, 
+        Returns all the ids belonging to proteins.
+        Inputs: 
+        * dataloader: a sidechainnet dataloader for a CASP version
+        Outputs: a set containing the ids for all protein entries. 
+    """
+    # store ids here
+    ids = set([])
+    # iterate for all batches
+    for i,batch in tqdm(enumerate(dataloaders['train'])):
+        # for breaking from 2 loops at once
+        try:
+            for i in range(batch.int_seqs.shape[0]):
+                # check if all fragments are : 4_LETTER_PDB + NUM + CHAIN
+                max_len_10 = len(batch.pids[i]) < 10 
+                fragments  = [len(x) <= 4 for x in batch.pids[i].split("_")] 
+                fragments_under_4 = sum(fragments) == len(fragments) # AND CONDITION
+                # record id 
+                if max_len_10 and fragments_under_4:
+                    ids.add(batch.pids[i])
+                else: 
+                    if verbose:
+                        print("skip:", batch.pids[i], "under 4", fragments)
+        except StopIteration:
+            break
+    # returns set of ids
+    return ids
+    
+
+def scn_cloud_mask(scn_seq, boolean=True, coords=None):
     """ Gets the boolean mask atom positions (not all aas have same atoms). 
         Inputs: 
         * scn_seq: (batch, length) sequence as provided by Sidechainnet package
         * boolean: whether to return as array of idxs or boolean values
+        * coords: optional .(batch, lc, 3). sidechainnet coords.
+                  returns the true mask (solves potential atoms that might not be provided)
         Outputs: (batch, length, NUM_COORDS_PER_RES) boolean mask 
     """
+
     scn_seq = expand_dims_to(scn_seq, 2 - len(scn_seq.shape))
+    # early check for coords mask
+    if coords is not None: 
+        batch_mask = ( rearrange(coords, '... (l c) d -> ... l c d', c=14) == 0 ).sum(dim=-1) < coords.shape[-1]
+        if boolean:
+            return batch_mask.bool()
+        else: 
+            return batch_mask.nonzero()
+
+    # do loop in cpu
     device = scn_seq.device
     batch_mask = []
-    # do loop in cpu
     scn_seq = scn_seq.cpu()
     for i, seq in enumerate(scn_seq):
         # get masks for each prot (points for each aa)
@@ -234,9 +289,13 @@ def scn_cloud_mask(scn_seq, boolean=True):
                                          for aa in seq]).bool().to(device).unsqueeze(0) )
     # concat in last dim
     batch_mask = torch.cat(batch_mask, dim=0)
+    # return mask (boolean or indexes)
     if boolean:
         return batch_mask.bool()
-    return batch_mask.nonzero()
+    else: 
+        return batch_mask.nonzero()
+
+    
 
 def scn_backbone_mask(scn_seq, boolean=True, n_aa=3):
     """ Gets the boolean mask for N and CA positions. 
